
import java.net.*;
import java.io.*;
import java.awt.*;
import java.awt.event.ActionEvent;

/** Simple pure-java version of the robot client component that implements only remote networking subset for scripting; Use this in applet that will control a local or remote robot through another robot component.
 *  the purpose is that the java applet in web browser cannot call native dll routines, and thus the robot.Robot cannot be used.
 * @see robot.Robot */
public class RemoteRobot implements Runnable, java.awt.event.ActionListener {

    /** server port where the robot component is listening */
    private	static final int ROBOT_SERVER_PORT = 7123;

    /** format of the packets transmitted between net objects */
    private static final int PACKET_HEADER_SIZE = 10;
    private static final int ROBOTNET_TAG   = 0;
    private static final int ROBOTNET_TYPE  = 1;
    private static final int ROBOTNET_ID    = 2;
    private static final int ROBOTNET_LEN   = 6;

	//last error msg
	public String errMsg;

	private int port;

	private Frame owner;  //needed for displaying messageboxes

	/** constructor does almost nothing */
	public RemoteRobot(Frame ownerFrame)
	{
		id_sent = 0;
		force_logoff = false;
		do_not_exclusive = false;
		s = null;
		port = ROBOT_SERVER_PORT;
		owner = ownerFrame;
	}

	/** constructor that allows to specify different robot daemon ip port */
	public RemoteRobot(Frame ownerFrame, int port)
	{
		id_sent = 0;
		force_logoff = false;
		do_not_exclusive = false;
		s = null;
		this.port = port;
		owner = ownerFrame;
	}

	private boolean force_logoff;
	private boolean do_not_exclusive;

	/** if busy, force logoff of the previous user (only for admin) when you connect */
	public void forceLogoff(boolean on)
	{
	  force_logoff = on;
	}

	/** if you call this before connecting, we will not login exclusively - so that multiple users can be logged in at the same time. */
	public void withoutExclusive(boolean state)
	{
	  do_not_exclusive = true;
	}

	/** connect to remote robot at specified port and create processing thread */
	public boolean connect(String remote_ip, String user, String password)
	{
	  username = user;
	  try { s.close(); } catch (Exception e) {} 
	  try { //System.out.println("Socket(" + remote_ip + ", "+ port + ")");
                s = new Socket(remote_ip, port); }
	  catch (UnknownHostException e)
	    { //System.out.println("Unknown host exception for " + remote_ip + ": " + e);
	      errMsg = "net?";
	      return false; }
	  catch (IOException e2)
	    { //System.out.println("IOException on openning socket: " + e2);
	      errMsg = "?net";
	      return false; }
          catch (Exception e3)
            {
              errMsg = "?NET";
	      //System.out.println("socket(): " + e3);
              return false; }
 	  // if (!s.isConnected()) return false;
 	  try
 	  {
 	    sin = s.getInputStream();
 	    sout = s.getOutputStream();
      } catch (IOException e)
        { //System.out.println("Exception getting input or output stream of socket: " + e);
		errMsg = "network?";
        }
 	  state = CONNECTED_LOGGING_IN;

 	  // System.out.println("Socket connected, sending username and password...");

 	  //start receiving packets
 	  Thread t = new Thread(this);
 	  t.start();

 	  //send LOGIN
	  try {
 	    if (false == send("/U0" + (char)(user.length() / 10 + (int)'0') +
 	                            (char)(user.length() % 10 + (int)'0') +
 	                            user + scramble(password, user)))
 	      state = LOGIN_FAILED;
	  } catch (IOException e3) { errMsg = "commerr"; try { s.close(); } catch (Exception ee3) {} s = null; return false; }

	  int login_timeout = 25000; /* if we cannot login in 15 seconds, report error */
	  int counter = 0;
 	  while ((state == CONNECTED_LOGGING_IN) && (counter * 50 < login_timeout))
 	    try { counter++; Thread.sleep(50); } catch (InterruptedException e) {}

 	  if (state != LOGGED_IN)
 	  {
		  //failed to login
		  try { s.close(); } catch (Exception e) {}
		  if (state == LOGIN_FAILED) errMsg = "denied";
		  else errMsg = "timeout";
		  return false;
	  }

      // System.out.println("Logged in, requesting exclusive access...");

	  //request EXCLUSIVE
	  if (do_not_exclusive == false)
	  {
	  try {
	  if (false == send("/E01" + user)) state = EXCLUSIVE_FAILED;
	  } catch (IOException e4) { state = EXCLUSIVE_FAILED; errMsg = "commerr"; try { s.close(); } catch (Exception ee4) {} s = null; return false; }

	  while ((state == LOGGED_IN) && (counter * 50 < login_timeout))
	    try { counter++; Thread.sleep(50); } catch (InterruptedException e) {}
	  }
	  else state = EXCLUSIVE_ACQUIRED;

	  if (state != EXCLUSIVE_ACQUIRED)
	  {
		  if (force_logoff)  //try to logoff previous user (only admin)
		  {
		      try {
                      if (false == send("/K0" + scramble(password, "admin")))
		      {
			      state = EXCLUSIVE_FAILED;
			      errMsg = "denied!";
			      return false;
		      }
		      } catch (IOException e5) { state = EXCLUSIVE_FAILED; errMsg = "commerr"; try { s.close(); } catch (Exception ee5) {} s = null; return false; }
		      state = EXCLUSIVE_ACQUIRED;
		      return true;
		  }
		  errMsg = "busy";
		  //failed to obtain exclusive access
		  try { s.close(); } catch (IOException e) {}
		  return false;
	  }

 	  return true;
    }

    /** robot should move forward n steps now */
    public void fd(short n) throws IOException
    {
		sendScript(" 0          fd " + n + "\n");
	}

    /** robot should move back n steps now */
    public void bk(short n) throws IOException
    {
		sendScript(" 0          bk " + n + "\n");
	}

    /** robot should move right n steps now */
    public void rt(short n) throws IOException
    {
		sendScript(" 0          rt " + n + "\n");
	}

    /** robot should move left n steps now */
    public void lt(short n) throws IOException
    {
		sendScript(" 0          lt " + n + "\n");
	}

    /** robot should move forward with given speed now */
    public void fdspeed(short l, short r) throws IOException
    {
		sendScript(" 0          fdspeed " + l + " " + r + "\n");
	}

    /** robot should move back with given speed now */
    public void bkspeed(short l, short r) throws IOException
    {
		sendScript(" 0          bkspeed " + l + " " + r + "\n");
	}

    /** robot should move right n steps now */
    public void rtspeed(short l, short r) throws IOException
    {
		sendScript(" 0          rtspeed " + l + " " + r + "\n");
	}

    /** robot should move left with given speed now */
    public void ltspeed(short l, short r) throws IOException
    {
		sendScript(" 0          ltspeed " + l + " " + r + "\n");
	}

    /** robot should move forward with given speed given number of steps now */
    public void fdx(short l, short r, short n) throws IOException
    {
		sendScript(" 0          fdx " + l + " " + r + " " + n + "\n");
	}

    /** robot should move back with given speed given number of steps now */
    public void bkx(short l, short r, short n) throws IOException
    {
		sendScript(" 0          bkx " + l + " " + r + " " + n + "\n");
	}

    /** robot should move right n steps given number of steps now */
    public void rtx(short l, short r, short n) throws IOException
    {
		sendScript(" 0          rtx " + l + " " + r + " " + n + "\n");
	}

    /** robot should move left with given speed given number of steps now */
    public void ltx(short l, short r, short n) throws IOException
    {
		sendScript(" 0          ltx " + l + " " + r + " " + n + "\n");
	}

    /** robot should move pen up */
    public void pu() throws IOException
    {
		sendScript(" 0          pu\n");
	}

    /** robot should move pen down */
    public void pd() throws IOException
    {
		try { Thread.sleep(1000); } catch (Exception e) {}
		sendScript(" 0          pd\n");
	}

	/** move robot gripper
	 * @param movement - one of the keywords "open", "close", "up", "down", "loose", "tighten", "upwards", "downwards", "fix" */
	public void gripper(String movement) throws IOException
	{
		sendScript(" 0          " + movement + "\n");
	}
	
	/** read sensors of remote robot (takes some time)
	 * @return sensory readings - binary combination, value 0-63
	 */
	public int sensors() throws IOException
	//note - this could be implemented asynchroneously (first send the request and then test if it arrived
	//  multiple times), or by polling sensors all the time automatically on the server and forwarding
	//  sensor packets all the time over the network so that the value is always available here
	//  all that remains for the future work.
	{
		byte check = (byte)(((byte)'?') ^ (byte)(((byte)'S')) ^ (byte)'0'); // ^ (byte)num));
		char c1, c2;
		if (check / 16 > 9) c1 = (char)(check / 16 - 10 + (byte)'A'); else c1 = (char)(check / 16 + (byte)'0');
		if (check % 16 > 9) c2 = (char)(check % 16 - 10 + (byte)'A'); else c2 = (char)(check % 16 + (byte)'0');

        sensors = -1;
        int waiting = 0;
		if (false == send("?S0" + c1 + c2)) connectionLost();
		else
		  while ((sensors < 0) && (waiting++ < 5000))
		    try { Thread.sleep(1); } catch (InterruptedException e) {}
		return sensors;
	}
	
        /** get position of the robot in the simulator
         * @return string of three numbers x y heading
         */
	public String globalpos(short reset) throws IOException
	{
	  String packet = "/O0" + reset;
	  String nopos = "0 0 0";
	  gpos = nopos;
	  if (false == send(packet))
		connectionLost();
  	  else {
		int waiting = 0;
		while ((gpos == nopos) && (waiting++ < 5000)) 
		{
			try { 
				Thread.sleep(1); 
			} catch (InterruptedException e) { }
	        }				
	  }
	  return gpos;
	}

	/** read ultrasonic sensors of remote robot (takes some time)
	 * @return sensory readings - 5 integer values 0-255
	 */
	public String distances() throws IOException
	//note - this could be implemented asynchroneously (first send the request and then test if it arrived
	//  multiple times), or by polling sensors all the time automatically on the server and forwarding
	//  sensor packets all the time over the network so that the value is always available here
	//  all that remains for the future work.
	{
		byte check = (byte)(((byte)'?') ^ (byte)(((byte)'G')) ^ (byte)'0'); // ^ (byte)num));
		char c1, c2;
		if (check / 16 > 9) c1 = (char)(check / 16 - 10 + (byte)'A'); else c1 = (char)(check / 16 + (byte)'0');
		if (check % 16 > 9) c2 = (char)(check % 16 - 10 + (byte)'A'); else c2 = (char)(check % 16 + (byte)'0');

                String no_distances = "-1 -1 -1 -1 -1";
                distances = no_distances;
		
                if (false == send("?G0" + c1 + c2)) {				
			connectionLost();
		} else {
			int waiting = 0;
			while ((distances == no_distances) && (waiting++ < 5000)) {
				try { 
					Thread.sleep(1); 
				} catch (InterruptedException e) {
				}
			}				
		}
		return distances;
	}

	/** read battery voltage of remote robot (takes some time)
	 * @return battery voltage in deciVolts
	 */
	public int battery() throws IOException
	{
		byte check = (byte)(((byte)'?') ^ (byte)(((byte)'V')) ^ (byte)'0'); // ^ (byte)num));
		char c1, c2;
		if (check / 16 > 9) c1 = (char)(check / 16 - 10 + (byte)'A'); else c1 = (char)(check / 16 + (byte)'0');
		if (check % 16 > 9) c2 = (char)(check % 16 - 10 + (byte)'A'); else c2 = (char)(check % 16 + (byte)'0');

        voltage = -1;
        int waiting = 0;
		if (false == send("?V0" + c1 + c2)) connectionLost();
		while ((voltage < 0) && (waiting++ < 5000))
		  try { Thread.sleep(1); } catch (InterruptedException e) {}
        return voltage;
	}

	/** determines whether command or script completed */
	public boolean completed()
	{
		if (state == SCRIPT_COMPLETED) return true;
		else return false;
	}

    /** wrap and send script message. Script should contain one command per line. Each line
      * contains time in milliseconds when the the command should be sent (or 0 if it should
      * be sent when previous command completes) - this should take 12 characters, and then
      * the command. Recognized commands are: <pre>
      *   fd n
      *   bk n
      *   lt n
      *   rt n
      *   fdspeed l r
      *   bkspeed l r
      *   ltspeed l r
      *   rtspeed l r
      *   fdx l r n
	  *   bkx l r n
	  *   ltx l r n
	  *   rtx l r n
	  *   pu
	  *   pd
	  *   setcond xormask ormask andmask
	  *   up
	  *   down
	  *   open
	  *   close
	  *   upwards
	  *   downwards
	  *   loose
	  *   tighten
	  *   fix
	  *   init
	  *  </pre>
	  * @see #completed*/
	public void sendScript(String s) throws IOException
	{
		if (send("/S0" + (char)((17 + s.length()) / 1000 + (int)'0') +
		                       (char)(((17 + s.length()) % 1000) / 100 + (int)'0') +
		                       (char)(((17 + s.length()) % 100) / 10 + (int)'0') +
		                       (char)(((17 + s.length()) % 10) + (int)'0') + s + " 0          nop\n"))
		                       //we added last nop so that the complete packet will come after last command finished
		  state = SCRIPT_SUBMITTED;
		else {
			state = SCRIPT_FAILED;
			connectionLost();
		}
	}

    /** stop the execution of the script immediatelly */
    public void stopscript() throws IOException
    {
		if (send("/P0") == false) connectionLost();
	}


    /** send packet to remote robot */
    private boolean send(String g) throws IOException
    {
		if (g == null) return false;

		try
		{
		  byte b0[] = g.getBytes();
		  byte b[] = new byte[b0.length + PACKET_HEADER_SIZE + 1];
		  for (int i = 0; i < b0.length; i++) b[i + PACKET_HEADER_SIZE] = b0[i];
		  b[b0.length + PACKET_HEADER_SIZE] = (byte)'\0';

		  //setup header
		  b[ROBOTNET_TAG] = (byte)'R';
		  b[ROBOTNET_TYPE] = (byte)'M';
		  ++id_sent;
		  b[ROBOTNET_ID] = (byte)('0' + id_sent / 1000);
		  b[ROBOTNET_ID + 1] = (byte)('0' + (id_sent % 1000) / 100);
		  b[ROBOTNET_ID + 2] = (byte)('0' + (id_sent % 100) / 10);
		  b[ROBOTNET_ID + 3] = (byte)('0' + (id_sent % 10));
		  b[ROBOTNET_LEN] = (byte)('0' + (b0.length + 1) / 1000);
  		  b[ROBOTNET_LEN + 1] = (byte)('0' + ((b0.length + 1) % 1000) / 100);
  		  b[ROBOTNET_LEN + 2] = (byte)('0' + ((b0.length + 1) % 100) / 10);
		  b[ROBOTNET_LEN + 3] = (byte)('0' + ((b0.length + 1) % 10));

		   //System.out.print("Sending the following message: '");
	      //for (int i = 0; i < b.length; i++) System.out.print((char)b[i]);
	      //System.out.print(" = ");
	      //for (int i = 0; i < b.length - 1; i++) System.out.print(b[i] + ", ");
	      //System.out.println(b[b.length - 1]);
		  
		  sout.write(b);
		} catch (IOException e)
		{
			try { s.close(); } catch (Exception e2) {}
			s = null;
			connectionLost(); //throws exception
		}
		return true;
    }

    private Dialog d;
    private Button butClose;
    /** message dialog close button */
    public void actionPerformed(ActionEvent e)
    {
      if (e.getSource() == butClose) 
      {
        d.dispose();
        d = null;
      }
    } 

    private void connectionLost() throws IOException
    {
	    messageFrom = "System";
	    message = "Not connected to the robot server. Connection has been lost or not established yet.";
	    showMessage("Attempt to communicate with the server failed");
	    throw new IOException("Not connected to the robot server");
    }

    public boolean sendMessage(String forUser, String message)
    {
      try {
      return send("/M0" + (char)(username.length() / 10 + (int)'0') +
	                            (char)(username.length() % 10 + (int)'0') +
		                    (char)(forUser.length() / 10 + (int)'0') +
	                            (char)(forUser.length() % 10 + (int)'0') +
                                     username + forUser + message + "\0");
      } catch (IOException e) { try { s.close(); } catch (Exception ee) {} s = null; return false; }
    }

    private void showMessage(String title)
    {
      if (d != null) d.dispose(); 
      d = new Dialog(owner, "Virtuallab: " + title, false);
      d.setLayout(new GridLayout(2,0));
      d.add(new Label(messageFrom + ": " + message));
      butClose = new Button("Close");
      butClose.addActionListener(this);
      d.add(butClose);
      d.pack();
      if (d.getX() == 0) d.setLocation(new Point(200,200));
      if (d.getWidth() < 200) d.setSize(200,100);
      d.show();
    }

    /** message handler */
    private void process_message()
    {
		switch (status)
		{
			case ROBOT_BUSY: state = LOGIN_FAILED;
			                 break;

            case MESSAGE_ARRIVED: if (owner != null) showMessage("Message from another user/system");
				  else System.out.println("Message: " + message);
                                  break;

            case ADMIN_KICK: state = EXCLUSIVE_FAILED;
                             exclusive_name = "admin";
                             if (owner != null)
			     {
				    message = "Administrator or a service program has closed our connection with the robot.";
				    messageFrom = "System";
				    showMessage("Connection has been closed");
			     }
			     else System.out.println("Connection has been closed by administrator or service program");
			     try { s.close(); } catch (Exception e) {}
			     s = null;
                             break;

            case SCRIPT_OK:	state = SCRIPT_STARTED_OK;
                            break;

            case SCRIPT_DONE: state = SCRIPT_COMPLETED;
            				  break;

            case SCRIPT_ERROR: state = SCRIPT_FAILED;
                               //error is in err
                               break;

			case EXCLUSIVE_OK: state = EXCLUSIVE_ACQUIRED;
			                   break;

			case EXCLUSIVE_DENIED: state = EXCLUSIVE_FAILED;
								   //name is in exclusive name
			                       break;

			case LOGIN_OK: state = LOGGED_IN;
			               break;

            case LOGIN_DENIED: state = LOGIN_FAILED;
                               break;
		}
	}

    /** this is the receiving thread main loop, called automatically */
    public void run() {	
	byte header[] = new byte[PACKET_HEADER_SIZE];
	int nread;
	byte m[];

	try { //catch IOException
		while (true) { //(s.isConnected()) 
			// read next packet header
			nread = 0;
			while (nread < PACKET_HEADER_SIZE) {
				int nreadnow = sin.read(header, nread, PACKET_HEADER_SIZE - nread);

				 /* lost connection */
				if (nreadnow == -1) {
					//System.out.println("nreadnow is -1");
			    		s = null;
			    		break;
			  	}
			  	nread += nreadnow;
			  	
				//System.out.println("nread = " + nread);	
				
  			   	//for (int xxx = 0; xxx < nread; xxx++) {
  			        //		System.out.println("h[" + xxx + "] = " + header[xxx] + " | " + (char)header[xxx]);
				//} 
		    	}
  			status = NO_MESSAGE;
			if (s == null) {
				break;
			}
			// System.out.println("header " + header);
  			// packet contains message
  			if ((header[ROBOTNET_TAG] == 'R') && (header[ROBOTNET_TYPE] == 'M')) {
				int ln = (header[ROBOTNET_LEN] - (byte)'0') * 1000 +
				         (header[ROBOTNET_LEN + 1] - (byte)'0') * 100 +
				         (header[ROBOTNET_LEN + 2] - (byte)'0') * 10 +
				         (header[ROBOTNET_LEN + 3] - (byte)'0');
				m = new byte[ln + 1];

				//System.out.println("packet, ln=" + ln);

				//read packet body
				nread = 0;
				int nr = 0;
				while ((nread < ln) && (nr >= 0)) {
 					nr = sin.read(m, nread, ln - nread);
					if (nr > 0) { 
						nread += nr;
					}
				}
				if (nr < 0) { 
					s = null; 
					break; 
				}

				/*
				System.out.println("packet " + ln);
				for (int yyy = 0; yyy < nread; yyy++) {
					System.out.println("p[" + yyy + "] = " + m[yyy] + " | " + (char)m[yyy]);
				}
				*/
 				//we are only interested in internal packets and sensor values, other robot packets are ignored
 				//System.out.println("m[0] = " + m[0]);
 				
				//DELETE:
				/*
				System.out.print("m = [");
				for (int i = 0; i < m.length; i++) {
					System.out.print("" + (char)m[i]);
									
				}
				System.out.println("]");
				*/
				
				if (m[0] == '/')
 				{
				  switch (m[1])	{

					case 'B':  // reply to our command: that the robot is busy by another user

						exclusive_name = new String(m, 3, ln - 3);
						status = ROBOT_BUSY;
						break;

					case 'M':  // message from another user

                        int fromlen = (m[3] - 48) * 10 + (m[4] - 48);
                        int tolen = (m[5] - 48) * 10 + (m[6] - 48);
                        String fromUser = new String(m, 7, fromlen);
                        String toUser = new String(m, 7 + fromlen, tolen);
                        if (toUser.equals(username))
                        {
						  messageFrom = fromUser;
						  message = new String(m, 7 + fromlen + tolen, ln - (7 + fromlen + tolen + 1)); 
						   //(last char is just terminating \0 for C strings, so we ignore that one)
						status = MESSAGE_ARRIVED;
					    }
						break;

					case 'K':   // kick-out from admin, end of exclusive mode

						status = ADMIN_KICK;
						break;

					case 'S':   // reply from a script request

					    if (m[3] == '0') status = SCRIPT_OK;
					    else status = SCRIPT_ERROR;
					    err = (char)(m[3]);
						break;

					case 'O':   // script execution complete

					    status = SCRIPT_DONE;
					    break;

					case 'E':   // must be confirm of exclusive request E2 or E3name

						if (m[3] == '2') status = EXCLUSIVE_OK;
					    else if (m[3] == '3')
						{
						    exclusive_name = new String(m, 4, ln - 4);
							status = EXCLUSIVE_DENIED;
						}
						break;

					case 'W':	// login request ok (Welcome)

					    status = LOGIN_OK;
					    num = (char)(m[2]); //store robot number
					    break;

					case 'D':   //login request access Denied

						status = LOGIN_DENIED;
						break;
					case 'P':   // global position info
						gpos = new String(m, 3, m.length - 3);
						break;

			      } //switch

				} //if internal
				else if (m[0] == 'S') //sensor state packet
				{
/*					if (m[2] > '9') sensors = 16 * (((int)(m[2])) - (int)'A' + 10); else sensors = 16 * (((int)m[2]) - (int)'0');
					if (m[3] > '9') sensors += ((int)(m[3])) - (int)'A' + 10; else sensors += ((int)m[3]) - (int)'0'; */
					sensors = 10 * (m[2] - '0') + m[3] - '0';
				}
				else if (m[0] == 'G') 
				{
					String s = "";
					for (int i = 0; i < 10; i += 2) {
						//System.out.println("" + (char)m[2 + i] + (char)m[2 + i + 1]);
						int n = Integer.parseInt("" + (char)m[2 + i] + (char)m[2 + i + 1], 16);
						s += n + "|";
					}
					distances = new String(s);
				}
				else if (m[0] == 'V') //voltage state packet
				{
					voltage = 10 * (m[2] - '0') + m[3] - '0';
				}

				process_message();

			} //if robotpacket

		  } //while connected

	    } catch (IOException e)
	    {
		//System.out.println(e.getMessage());
		}

	          try { s.close(); } catch (Exception e) {}
  	    //System.out.println("receiving thread terminated.");
  	    s = null;
	}

	/** close the socket */
    public void close()
    {
			try { 
			  send("/E00" + username); 
			  s.close(); 
			} catch (Exception e) {}
			s = null;
	}

    /** scramble password so that it won't be plain text */
	private String scramble(String p, String k)
	{
		if ((p == null) || (k == null)) return "";
  	    if (p.length() == 0) return "";
        if (k.length() == 0) return p;

        StringBuffer sp = new StringBuffer(p);
		int j = k.length();

		for (int i = 0; i < p.length(); i++)
        {
	        sp.setCharAt(i, (char)(((byte)32) | (byte)(p.charAt(i)) ^ (byte)(k.charAt(--j))));
	        if (j == 0) j = k.length();
		}
		return sp.toString();
    }

	/** client socket that is connected to the server */
    private	Socket s;

	/** socket input stream */
	private InputStream sin;

	/** socket output stream */
	private OutputStream sout;

	/** receiving thread */
	private Thread t;

	/** username we used to login */
	private String username;

	/** state of the communication */
	private short state;

	/** robot number */
	private char num;

	private int id_sent; //packet id

	/** state constants */
	private static final short CONNECTED_LOGGING_IN = 1;
	private static final short LOGGED_IN            = 2;
	private static final short LOGIN_FAILED         = 3;
	private static final short EXCLUSIVE_ACQUIRED   = 4;
	private static final short EXCLUSIVE_FAILED     = 5;
	private static final short SCRIPT_SUBMITTED		= 6;
	private static final short SCRIPT_STARTED_OK    = 7;
	private static final short SCRIPT_COMPLETED		= 8;
	private static final short SCRIPT_FAILED        = 9;

	/** message status */
	private short status;

	/** status constants */
	private static final short NO_MESSAGE       = 0;
	private static final short ROBOT_BUSY       = 1;
	private static final short MESSAGE_ARRIVED  = 2;
	private static final short ADMIN_KICK       = 3;
	private static final short SCRIPT_OK        = 4;
	private static final short SCRIPT_ERROR     = 5;
	private static final short SCRIPT_DONE      = 6;
	private static final short EXCLUSIVE_OK     = 7;
	private static final short EXCLUSIVE_DENIED = 8;
	private static final short LOGIN_OK         = 9;
	private static final short LOGIN_DENIED     = 10;

	private char err;  //script error code
	private String message; //message from another user
	private String messageFrom; //name of the user who sent a message
	private String exclusive_name; //name of the user who is blocking us with exclusive access

	private int sensors;  // sensor status
	private int voltage;  // battery status
	private String distances; // ultrasonic sensors readings
	public String gpos;     // global position of robot received from server
}

